"use strict";
// Extract interface from a class declaration. Write it just before the class declaration and make the class implement it. Only members with "public" modifier will be extracted
//
// **Screencast**: 
// 
// ![See it in action](../plugin-screencast.gif)
const typescript_ast_util_1 = require("typescript-ast-util");
// import * as ts from 'typescript'
const extract_interface_1 = require("./extract-interface");
const hrtime_now_1 = require("hrtime-now");
const PLUGIN_NAME = 'typescript-plugin-extract-interface';
const REFACTOR_ACTION_NAME = `${PLUGIN_NAME}-refactor-action`;
let ts;
let info;
function init(modules) {
    ts = modules.typescript;
    function create(anInfo) {
        info = anInfo;
        info.project.projectService.logger.info(`${PLUGIN_NAME} created`);
        const proxy = Object.create(null);
        for (let k of Object.keys(info.languageService)) {
            const x = info.languageService[k];
            proxy[k] = (...args) => x.apply(info.languageService, args);
        }
        proxy.getApplicableRefactors = getApplicableRefactors;
        proxy.getEditsForRefactor = getEditsForRefactor;
        return proxy;
    }
    return { create };
}
let selectedDef;
function getApplicableRefactors(fileName, positionOrRange, preferences) {
    const t0 = hrtime_now_1.now();
    const refactors = info.languageService.getApplicableRefactors(fileName, positionOrRange, preferences) || [];
    const refs = info.languageService.findReferences(fileName, typescript_ast_util_1.positionOrRangeToNumber(positionOrRange));
    if (!refs || !refs.length) {
        return refactors;
    }
    const selectedDefs = refs.filter(r => r.definition && r.definition.kind === ts.ScriptElementKind.classElement && r.definition.fileName === fileName);
    if (!selectedDefs || !selectedDefs.length) {
        return refactors;
    }
    selectedDef = selectedDefs[0];
    refactors.push({
        name: `${PLUGIN_NAME}-refactor-info`,
        description: 'Extract interface',
        actions: [
            { name: REFACTOR_ACTION_NAME, description: 'Extract interface from ' + selectedDef.definition.name }
        ],
    });
    info.project.projectService.logger.info(`${PLUGIN_NAME} getApplicableRefactors took ${hrtime_now_1.now() - t0}`);
    return refactors;
}
function getEditsForRefactor(fileName, formatOptions, positionOrRange, refactorName, actionName, preferences) {
    const t0 = hrtime_now_1.now();
    const refactors = info.languageService.getEditsForRefactor(fileName, formatOptions, positionOrRange, refactorName, actionName, preferences);
    if (actionName != REFACTOR_ACTION_NAME) {
        return refactors;
    }
    const sourceFile = info.languageService.getProgram().getSourceFile(fileName);
    // find the first parent that is a class declaration starting from given position
    const targetNode = typescript_ast_util_1.findParentFromPosition(sourceFile, positionOrRange, parent => parent.kind === ts.SyntaxKind.ClassDeclaration);
    if (targetNode && ts.isClassDeclaration(targetNode)) {
    }
    const newText = extract_interface_1.extractInterface(targetNode);
    if (targetNode && newText) {
        const targetSourceFile = targetNode.getSourceFile();
        info.project.projectService.logger.info(`${PLUGIN_NAME} getEditsForRefactor took ${hrtime_now_1.now() - t0}`);
        return {
            edits: [{
                    fileName,
                    textChanges: [{
                            span: { start: targetSourceFile.getEnd(), length: newText.length },
                            newText: newText
                        }],
                }],
            renameFilename: undefined,
            renameLocation: undefined,
        };
    }
    else {
        return refactors;
    }
}
module.exports = init;
